include <poly-definitions.scad>

//This module describes the box of the spectrograph. Change the following to customize:
//
//		1st ; the angle of deviation of the 1st order diffracted light
//	l1l, l2l; the length of the two boxes holding the lenses
//	lw, lh	; the width and height of the boxes
//
module poly(){
	difference(){
		linear_extrude(height=lh, center=true, convexity = 10) 
			polygon(points = [p1, p2, p3, p4, p5, p6, p7], convexity = 10);

		linear_extrude(height=lh-6, center=true, convexity = 10) 
			polygon(points = [p1i, p2i, p3i, p4i, p5i, p6i, p7i], convexity = 10);			
	}
}

//This module draws the rim to make the top and bottom more light proof when
//the entire spectrometer is assembled. The coordinates are defined relative to 
//the module poly(), so no need to change anything. The rim width is defined by rw
//
module lid_rim(){
//	difference(){
//		linear_extrude(height=2, center=true, convexity = 10) 
//			polygon(points = [p1ro, p2ro, p3ro, p4ro, p5ro, p6ro, p7ro], convexity = 10);

		linear_extrude(height=2, center=true, convexity = 10) 
			polygon(points = [p1ri, p2ri, p3ri, p4ri, p5ri, p6ri, p7ri], convexity = 10);			
//	}
}

//This module draws the lens holders. Change the following:
//
// Variable	; Function
//	pentaxr	; the radius of the front lens
//		M3	; the radius of the holes for the screws to secure the lenses
//		ls	; coordinates of the screws (must match the plate the lens is attached to
//
module lens_holder(){
	difference(){
		cube(size=[lht,lw,lh], center=true);
		rotate([90,22.5,90]) cylinder(r=pentaxr, h=4, $fn=8, center=true);

		translate([0,lw/2-ls,lh/2-ls-2]) rotate([0,90,0]) cylinder(r=M3, h=4, center=true);
		translate([0,lw/2-ls,-lh/2+ls]) rotate([0,90,0]) cylinder(r=M3, h=4, center=true);
		translate([0,-lw/2+ls,lh/2-ls-2]) rotate([0,90,0]) cylinder(r=M3, h=4, center=true);
		translate([0,-lw/2+ls,-lh/2+ls]) rotate([0,90,0]) cylinder(r=M3, h=4, center=true);

	}
}

//grating-slide holder. Change gw and gh if you use a slide-mounted film grating.
module grating_holder(){
	difference(){
		cube(size=[ghw,lw,lh]);
		translate([ghw/2-gw/2,2.5,2.5]) cube(size=[gw,lw-5,lh-5]);
		translate([0,lw/2-40/2,10]) cube(size=[ghw+1,40,50]);
	}
}

//baffles for the port
module baffles(){
	difference(){
		cube(size=[1.5,lw,lh], center=true);
		cube(size=[2,lw-20,lh-15], center=true);
	}
}

//screw stand-off for assembling
module M3_standoff(){
	difference(){
		cube(size=[7,8,lh]);
		translate([3.5,4.5,lh/2]) cylinder(r=M3, h=lh, $fn=6, center=true);
	}
}
module M3_through(){
		translate([3.5,4.5,lh/2]) cylinder(r=M3, h=lh, $fn=6, center=true);
}


//Connect all the pieces
rotate([0,180,1st]) difference(){
	union(){
		//draw the spectrograph
		poly();

		//insert lens holders
		//ff is the distance from the focal plane to the front of the lens
		translate([l1l-ghw-1,lw/2,0]) lens_holder();
		translate([l1l+cos(90+1st)*lw/2+cos(1st)*1,sin(90+1st)*lw/2+sin(1st)*1,0]) rotate([0,0,1st]) lens_holder();

		//add baffles
		translate([12,lw/2,0]) baffles();
		
		//draw the box for the Nucleo
		translate([l1l-cos(90+1st)*(mcuw-1)+cos(1st)*(l2l-mcul),-sin(90+1st)*(mcuw-1)+sin(1st)*(l2l-mcul),-lh/2]) rotate([0,0,1st]) mcu_rpi();

		//draw grating-slide holder
		translate([l1l-ghw,0,-lh/2]) grating_holder();

		//draw screw stand-offs
		translate([l1l/3,wt,-lh/2]) M3_standoff();
		translate([l1l/3,lw-10,-lh/2]) M3_standoff();
		translate([l1l+cos(1st)*(l2l-l2l/2)+cos(90+1st)*wt,sin(1st)*(l2l-l2l/2)+sin(90+1st)*wt,-lh/2]) rotate([0,0,1st]) M3_standoff();
		translate([l1l+cos(1st)*(l2l-l2l/2)+cos(90+1st)*(lw-10),sin(1st)*(l2l-l2l/2)+sin(90+1st)*(lw-10),-lh/2]) rotate([0,0,1st]) M3_standoff();

	}


	//install rim for the spectrograph
	translate([0,0,lh/2-10]) lid_rim();

	translate([0,-200,-lh+lh/2-11]) cube(size=[400,400,lh]);


		translate([l1l/3,wt,-lh/2]) M3_through();
		translate([l1l/3,lw-10,-lh/2]) M3_through();
		translate([l1l+cos(1st)*(l2l-l2l/2)+cos(90+1st)*wt,sin(1st)*(l2l-l2l/2)+sin(90+1st)*wt,-lh/2]) rotate([0,0,1st]) M3_through();
		translate([l1l+cos(1st)*(l2l-l2l/2)+cos(90+1st)*(lw-10),sin(1st)*(l2l-l2l/2)+sin(90+1st)*(lw-10),-lh/2]) rotate([0,0,1st]) M3_through();

	
}
